--<< ================================================= >>--
--    Initialize the AddOn Object                        --
--<< ================================================= >>--
local L = AceLibrary("AceLocale-2.2"):new("Visor2GUI")
local _G = getfenv(0)


Visor2GUI = Visor2:NewModule("GUI", "AceConsole-2.0", "AceDebug-2.0")

Visor2GUI.options = {
	type = 'group',
	handler = Visor2GUI,
	args = {
		toggle = {
			name = "Toggle Visor2 GUI Frame",
			type = 'toggle',
			desc = "This option allows you to toggle the Visor2 GUI window",
			get = function()
				return Visor2GUIFrame:IsShown()
			end,
			set = "Toggle",
		},
	},
}

Visor2GUI.locals = {
	DelConfirm = L["Are you sure you want to delete this frame's settings?"],
	DelConfirmCont = L["Deleting this frame will take effect on the next reload."],
	FrameEditBox = L["Enter Frame Name or Select Frame with Grab button"],
	GrabFrame = L["Drag Mouse and release on target Frame"],

	NudgeLeftTip = L["Nudge Left"],
	NudgeRightTip = L["Nudge Right"],
	NudgeUpTip = L["Nudge Up"],
	NudgeDownTip = L["Nudge Down"],
	WidthUpTip = L["Width Up"],
	WidthDownTip = L["Width Down"],
	HeightUpTip = L["Height Up"],
	HeightDownTip = L["Height Down"],

	GrabTip = L["Left-Click and hold the mouse button, move the mouse over a frame and release the button to select that frame"],
	GrabParentTip = L["Set this parent as the active frame"],
	DoneTip = L["Exit Visor2 GUI"],
	DelTip = L["Delete the selected frame's settings from Visor2"],
	HideTip = L["Hide the selected frame"],
	ScaleSliderTip = L["Adjust frame scale"],
	AlphaSliderTip = L["Adjust frame alpha"],
	EditWidthTip = L["Enter Width"],
	EditHeightTip = L["Enter Height"],
	EditXTip = L["Enter X Coordinate (Center point of frame)"],
	EditYTip = L["Enter Y Coordinate (Center point of frame)"],
	VirtTip = L["Show a virtual copy of a hidden frame."],
}


-- Rounding code stolen from Ace1
local function round(num)
	num = tonumber(num) or 0
	return floor(num + .5)
end

--<< ================================================= >>--
--    En/Disable, Hooks, Events, Et Cetera               --
--<< ================================================= >>--
function Visor2GUI:OnEnable()
	tinsert(UISpecialFrames, "Visor2GUIFrame")
	tinsert(UISpecialFrames, "Visor2GUIFrameConfirm")

	self.debugFrame = ChatFrame1
	self:SetDebugging(false)

	self:RegisterEvent("VISOR_UPDATE")
	self:RegisterChatCommand({"/visorgui", "/vzg"}, self.options)
end

--<< ================================================= >>--
--    Main Functions                                     --
--<< ================================================= >>--
function Visor2GUI:VISOR_UPDATE(p)
	if Visor2GUIFrame:IsVisible() then
		if p.f then
			if self.virtual then
				if p.f == "Visor2GUIVirtualFrame" or p.f == self.virtual then
					self.f = self.virtual
				end
			else
				self.f = p.f
			end

			local parent = _G[self.f]:GetParent()
			if parent then
				self.parent = parent:GetName()
			end

			if self.parent then
				Visor2GUIFrameTargetFrameLabel:SetText("Parent Frame:  |cffffffff"..self.parent)
			end

			Visor2GUIFrameEditBox:SetText(self.f)

			if _G[self.f].GetScale then
				local s = p.s or _G[self.f]:GetScale()
				Visor2GUIFrameScaleSlider:SetValue(s)
			end

			local a = p.a or _G[self.f]:GetAlpha()
			Visor2GUIFrameAlphaSlider:SetValue(a)

			local wh = p.wh or _G[self.f]:GetWidth()
			wh = round(wh)
			Visor2GUIFrameEditWidth:SetText(wh)

			local ht = p.ht or _G[self.f]:GetHeight()
			ht = round(ht)
			Visor2GUIFrameEditHeight:SetText(ht)

			local x, y = _G[self.f]:GetCenter()
			x = p.x or round(x)
			y = p.y or round(y)
			Visor2GUIFrameEditX:SetText(x)
			Visor2GUIFrameEditY:SetText(y)

			if _G[p.f]:IsVisible() then
				Visor2GUIFrameHideButton:SetText("Hide Frame")
			else
				Visor2GUIFrameHideButton:SetText("Show Frame")
			end

			self:Debug("Frame is: "..p.f)
		end
	end
end

function Visor2GUI:FrameEditBoxUpdate()
	if self.virtual then
		Visor2GUIFrameEditBox:SetText(self.virtual)
	else
		Visor2:SetupFrame("f="..this:GetText())
	end
end

function Visor2GUI:EditWUpdate()
	local n = Visor2GUIFrameEditHeight:GetText()

	if self.virtual then
		Visor2:Do("f="..self.virtual.." wh="..n)
		Visor2:Do("f=Visor2GUIVirtualFrame wh="..n)
	else
		Visor2:Do("wh="..n)
	end
end

function Visor2GUI:ButtonWUp()
	local n = Visor2GUIFrameEditWidth:GetText() + 1

	if self.virtual then
		Visor2:Do("f="..self.virtual.." wh="..n)
		Visor2:Do("f=Visor2GUIVirtualFrame wh="..n)
	else
		Visor2:Do("wh="..n)
	end
end

function Visor2GUI:ButtonWDown()
	local n = Visor2GUIFrameEditWidth:GetText() - 1

	if self.virtual then
		Visor2:Do("f="..self.virtual.." wh="..n)
		Visor2:Do("f=Visor2GUIVirtualFrame wh="..n)
	else
		Visor2:Do("wh="..n)
	end
end

function Visor2GUI:EditHUpdate()
	local n = Visor2GUIFrameEditHeight:GetText()

	if self.virtual then
		Visor2:Do("f="..self.virtual.." ht="..n)
		Visor2:Do("f=Visor2GUIVirtualFrame ht="..n)
	else
		Visor2:Do("ht="..n)
	end
end

function Visor2GUI:ButtonHUp()
	local n = Visor2GUIFrameEditHeight:GetText() + 1

	if self.virtual then
		Visor2:Do("f="..self.virtual.." ht="..n)
		Visor2:Do("f=Visor2GUIVirtualFrame ht="..n)
	else
		Visor2:Do("ht="..n)
	end
end

function Visor2GUI:ButtonHDown()
	local n = Visor2GUIFrameEditHeight:GetText() - 1

	if self.virtual then
		Visor2:Do("f="..self.virtual.." ht="..n)
		Visor2:Do("f=Visor2GUIVirtualFrame ht="..n)
	else
		Visor2:Do("ht="..n)
	end
end

function Visor2GUI:ScaleUpdate()
	local s = this:GetValue()

	if self.virtual then
		Visor2:Do("f="..self.virtual.." s="..s)
		Visor2:Do("f=Visor2GUIVirtualFrame s="..s)
	else
		Visor2:Do("s="..s)
	end
end

function Visor2GUI:AlphaUpdate()
	local a = this:GetValue()

	if self.virtual then
		Visor2:Do("f="..self.virtual.." a="..a)
		Visor2:Do("f=Visor2GUIVirtualFrame a="..a)
	else
		Visor2:Do("a="..a)
	end
end

function Visor2GUI:NudgeUpdate()
	local n = this:GetValue()
	Visor2:Do("n="..n)
end

function Visor2GUI:EditXUpdate()
	local x = Visor2GUIFrameEditX:GetText()

	if self.virtual then
		Visor2:Do("f="..self.virtual.." x="..x)
		Visor2:Do("f=Visor2GUIVirtualFrame x="..x)
	else
		Visor2:Do("x="..x)
	end

	Visor2GUIFrameEditX:SetText(x)
end

function Visor2GUI:EditYUpdate()
	local y = Visor2GUIFrameEditY:GetText()

	if self.virtual then
		Visor2:Do("f="..self.virtual.." y="..y)
		Visor2:Do("f=Visor2GUIVirtualFrame y="..y)
	else
		Visor2:Do("y="..y)
	end

	Visor2GUIFrameEditY:SetText(y)
end

function Visor2GUI:NudgeLeft()
	if self.virtual then
		Visor2:Do("f="..self.virtual)
		Visor2Nudge:Left()

		Visor2:Do("f=Visor2GUIVirtualFrame")
		Visor2Nudge:Left()
	else
		Visor2Nudge:Left()
	end
end

function Visor2GUI:NudgeRight()
	if self.virtual then
		Visor2:Do("f="..self.virtual)
		Visor2Nudge:Right()

		Visor2:Do("f=Visor2GUIVirtualFrame")
		Visor2Nudge:Right()
	else
		Visor2Nudge:Right()
	end
end

function Visor2GUI:NudgeUp()
	if self.virtual then
		Visor2:Do("f="..self.virtual)
		Visor2Nudge:Up()

		Visor2:Do("f=Visor2GUIVirtualFrame")
		Visor2Nudge:Up()
	else
		Visor2Nudge:Up()
	end
end

function Visor2GUI:NudgeDown()
	if self.virtual then
		Visor2:Do("f="..self.virtual)
		Visor2Nudge:Down()

		Visor2:Do("f=Visor2GUIVirtualFrame")
		Visor2Nudge:Down()
	else
		Visor2Nudge:Down()
	end
end

function Visor2GUI:DelFrame()
	Visor2:DeleteFrame(self.f)
end

function Visor2GUI:ButtonVirt()
	local f = self.f
	local vf = _G["Visor2GUIVirtualFrame"]

	if vf:IsVisible() then
		vf:Hide()
		self.virtual = nil
		self:Debug("Virtual Frame: nil")
		return
	end

	if _G[f] then
		self.virtual = _G[f]:GetName()
		self:Debug("Virtual Frame: "..self.virtual)

		_G["Visor2GUIVirtualFrameTitleString"]:SetText(f)
		Visor2:Do("f=Visor2GUIVirtualFrame")

		local h = _G[f]:GetHeight() or 200
		local w = _G[f]:GetWidth() or 200
		local s = _G[f]:GetScale() or 1
		local x, y = _G[f]:GetCenter()
		local p = _G[f]:GetParent()

		if p then
			p = _G[f]:GetParent():GetName()
		else
			p = "UIParent"
		end

		if not x then x = 500 end
		if not y then y = 300 end

		vf:SetParent(p)

		if not vf.VisorSetScale then
			vf:SetScale(s)
		else
			vf:VisorSetScale(s)
		end

		if not vf.VisorSetPoint then
			vf:SetPoint("CENTER", p, "BOTTOMLEFT", x, y)
		else
			vf:VisorSetPoint("CENTER", p, "BOTTOMLEFT", x, y)
		end

		Visor2:Do("f=Visor2GUIVirtualFrame s="..s.." p1=CENTER p2=BOTTOMLEFT")
		Visor2:Do("f=Visor2GUIVirtualFrame x="..x.." y="..y.." p="..p.." pr="..p)

		if h and w then
			Visor2:Do("f=Visor2GUIVirtualFrame".." ht="..h.." wh="..w)
		end

		vf:Show()
	end
end

function Visor2GUI:Toggle()
	if Visor2GUIFrame:IsShown() then
		Visor2GUIFrame:Hide()
	else
		Visor2GUIFrame:Show()
	end
end

-- Rounding function with precision, ace.round doesn't let you specify how many decimal places
function Visor2GUI:VRound(num, shift)
	shift = 10^shift
	return floor((num * shift) + .5) / shift
end
